import java.awt.*;
import java.applet.Applet;

public class Person implements ActingShape
{
   // these fields are protected so that other 
   // non-related classes cannot access them directly.  
   // Thus, the only way to make a person move its
   // hands would be to call raiseLeftHand(), etc.
   
   // Note that we use protected instead of private, though,
   // so that subclasses of Person can access these fields
   
   // properties of the Person
   
   protected int headRadius = 10;
   protected int bodyLength = 40;
   protected int armWidth = 20;
   protected int armLength = 20;
   protected int legWidth = 20;
   protected int legLength = 30;
   
   // parts of the Person
   
   protected int x = armWidth;
   protected int y = headRadius;
   
   protected Point rightHand, leftHand;
   protected Point rightFoot, leftFoot;
   protected Point neck, shoulder, pelvis;
   
   // state of the Person (used in PersonApplet2d)
   protected int state = 0;
   
   
   public Person()
   {
      setXY( armWidth, headRadius );
   }
   
   public Person( int x, int y )
   {
      setXY( x, y );
   }
   
   public Person( int x, int y,
                  int headRadius, int bodyLength,
                  int armWidth, int armLength,
                  int legWidth, int legLength )
   {
      this.headRadius = headRadius;
      this.bodyLength = bodyLength;
      this.armWidth = armWidth;
      this.armLength = armLength;
      this.legWidth = legWidth;
      this.legLength = legLength;
      setXY( x, y );
   }
   
   public void setXY( int x, int y )
   {
      this.x = x;
      this.y = y;
      
      this.neck = new Point( x, y + headRadius );
      this.shoulder = new Point( x, neck.y + ( bodyLength / 3 ) );
      this.pelvis = new Point( x, neck.y + bodyLength );
      
      
      this.rightHand = new Point( x + armWidth, this.shoulder.y + armLength );
      this.leftHand = new Point( x - armWidth, this.shoulder.y + armLength );
      
      this.rightFoot = new Point( x + legWidth, this.pelvis.y + legLength );
      this.leftFoot = new Point( x - legWidth, this.pelvis.y + legLength );
   }
   
   public int getX()
   {
      return this.x;
   }
   
   public int getY()
   {
      return this.y;
   }
   
   // convenience method
   
   protected void drawLine( Graphics g, Point p1, Point p2 )
   {
      g.drawLine( p1.x, p1.y, p2.x, p2.y );
   }
   
   public void draw( Graphics g ) 
   {  
      g.drawOval( this.x - headRadius, this.y - headRadius,
                  headRadius * 2, headRadius * 2 );
      drawLine( g, neck, pelvis );
      drawLine( g, shoulder, leftHand );
      drawLine( g, shoulder, rightHand );
      drawLine( g, pelvis, leftFoot );
      drawLine( g, pelvis, rightFoot );
   }
   
   // Person's actions ...
   
   public void raiseLeftHand()
   {
      leftHand.y = shoulder.y - armLength;
   }
   
   public void lowerLeftHand()
   {
      leftHand.y = shoulder.y + armLength;
   }
   
   public void raiseRightHand()
   {
      rightHand.y = shoulder.y - armLength;
   }
   
   public void lowerRightHand()
   {
      rightHand.y = shoulder.y + armLength;
   }
   
   // add more methods here ...
   
   public void raiseLeftFoot()
   {
      leftFoot.y = pelvis.y + ( legLength / 2 );
   }
   
   public void lowerLeftFoot()
   {
      leftFoot.y = pelvis.y + legLength;
   }
   
   public void raiseRightFoot()
   {
      rightFoot.y = pelvis.y + ( legLength / 2 );
   }
   
   public void lowerRightFoot()
   {
      rightFoot.y = pelvis.y + legLength;
   }
   
   public void moveHipLeft()
   {
      pelvis.x = this.x - ( legWidth / 2 );
   }
   
   public void moveHipRight()
   {
      pelvis.x = this.x + ( legWidth / 2 );
   }
   
   public void moveHipCenter()
   {
      pelvis.x = this.x;
   }
   
   // the act() method (used in PersonApplet 2d)
   
   /**
    * Change the body of this method to define what Person
    * does in response to a Button press.
    * (This assumes that the applet calls each Person's act()
    * method upon a Button press.)
    */
   public void act()
   {
      // go to next state, wrap-around if necessary
      state = ( state + 1 ) % 2;

      switch ( state )       
      {
         case 0: lowerLeftHand();
                 break;
         case 1: raiseLeftHand();
                 break;
      }
   }
}
